@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -21,63 +21,19 @@
 
 #include "DatabaseManager.h"
 
-#include <Plugins/Samples/Common/OrthancPluginCppWrapper.h>
+#include "Integer64Value.h"
+#include "BinaryStringValue.h"
+#include "Utf8StringValue.h"
+#include "../../Resources/Orthanc/Plugins/OrthancPluginCppWrapper.h"
 
-#include <Core/Logging.h>
-#include <Core/OrthancException.h>
+#include <Compatibility.h>  // For std::unique_ptr<>
+#include <Logging.h>
+#include <OrthancException.h>
 
 #include <boost/thread.hpp>
 
 namespace OrthancDatabases
 {
-  IDatabase& DatabaseManager::GetDatabase()
-  {
-    static const unsigned int MAX_CONNECTION_ATTEMPTS = 10;   // TODO: Parameter
-
-    unsigned int count = 0;
-      
-    while (database_.get() == NULL)
-    {
-      transaction_.reset(NULL);
-
-      try
-      {
-        database_.reset(factory_->Open());
-      }
-      catch (Orthanc::OrthancException& e)
-      {
-        if (e.GetErrorCode() == Orthanc::ErrorCode_DatabaseUnavailable)
-        {
-          count ++;
-
-          if (count <= MAX_CONNECTION_ATTEMPTS)
-          {
-            LOG(WARNING) << "Database is currently unavailable, retrying...";
-            boost::this_thread::sleep(boost::posix_time::seconds(1));
-            continue;
-          }
-          else
-          {
-            LOG(ERROR) << "Timeout when connecting to the database, giving up";
-          }
-        }
-
-        throw;
-      }
-    }
-
-    if (database_.get() == NULL ||
-        database_->GetDialect() != dialect_)
-    {
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
-    }
-    else
-    {
-      return *database_;
-    }
-  }
-
-
   void DatabaseManager::Close()
   {
     LOG(TRACE) << "Closing the connection to the database";
@@ -105,7 +61,16 @@ namespace OrthancDatabases
     
   void DatabaseManager::CloseIfUnavailable(Orthanc::ErrorCode e)
   {
-    if (e != Orthanc::ErrorCode_Success)
+    bool failure;
+
+#if ORTHANC_PLUGINS_VERSION_IS_ABOVE(1, 9, 2)
+    failure = (e != Orthanc::ErrorCode_Success &&
+               e != Orthanc::ErrorCode_DatabaseCannotSerialize);
+#else
+    failure = (e != Orthanc::ErrorCode_Success);
+#endif
+
+    if (failure)
     {
       transaction_.reset(NULL);
     }
@@ -139,7 +104,7 @@ namespace OrthancDatabases
   {
     LOG(TRACE) << "Caching statement from " << location.GetFile() << ":" << location.GetLine();
       
-    std::auto_ptr<IPrecompiledStatement> statement(GetDatabase().Compile(query));
+    std::unique_ptr<IPrecompiledStatement> statement(GetDatabase().Compile(query));
       
     IPrecompiledStatement* tmp = statement.get();
     if (tmp == NULL)
@@ -162,7 +127,7 @@ namespace OrthancDatabases
 
       try
       {
-        transaction_.reset(GetDatabase().CreateTransaction(true));
+        transaction_.reset(GetDatabase().CreateTransaction(TransactionType_Implicit));
       }
       catch (Orthanc::OrthancException& e)
       {
@@ -197,22 +162,57 @@ namespace OrthancDatabases
   }
 
     
-  DatabaseManager::DatabaseManager(IDatabaseFactory* factory) :  // Takes ownership
-    factory_(factory)
+  DatabaseManager::DatabaseManager(IDatabaseFactory* factory) :
+    factory_(factory),
+    dialect_(Dialect_Unknown)
   {
     if (factory == NULL)
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_NullPointer);
     }
-
-    dialect_ = factory->GetDialect();
   }
 
   
-  void DatabaseManager::StartTransaction()
+  IDatabase& DatabaseManager::GetDatabase()
   {
-    boost::recursive_mutex::scoped_lock lock(mutex_);
+    assert(factory_.get() != NULL);
+    
+    if (database_.get() == NULL)
+    {
+      database_.reset(factory_->Open());
 
+      if (database_.get() == NULL)
+      {
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      }
+
+      dialect_ = database_->GetDialect();
+      if (dialect_ == Dialect_Unknown)
+      {
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      }
+    }
+
+    return *database_;
+  }
+
+
+  Dialect DatabaseManager::GetDialect() const
+  {
+    if (database_.get() == NULL)
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+    }
+    else
+    {
+      assert(dialect_ != Dialect_Unknown);
+      return dialect_;
+    }
+  }
+  
+
+  void DatabaseManager::StartTransaction(TransactionType type)
+  {
     try
     {
       if (transaction_.get() != NULL)
@@ -221,7 +221,7 @@ namespace OrthancDatabases
         throw Orthanc::OrthancException(Orthanc::ErrorCode_Database);
       }
 
-      transaction_.reset(GetDatabase().CreateTransaction(false));
+      transaction_.reset(GetDatabase().CreateTransaction(type));
     }
     catch (Orthanc::OrthancException& e)
     {
@@ -233,8 +233,6 @@ namespace OrthancDatabases
 
   void DatabaseManager::CommitTransaction()
   {
-    boost::recursive_mutex::scoped_lock lock(mutex_);
-
     if (transaction_.get() == NULL)
     {
       LOG(ERROR) << "Cannot commit a non-existing transaction";
@@ -258,11 +256,9 @@ namespace OrthancDatabases
 
   void DatabaseManager::RollbackTransaction()
   {
-    boost::recursive_mutex::scoped_lock lock(mutex_);
-
     if (transaction_.get() == NULL)
     {
-      LOG(ERROR) << "Cannot rollback a non-existing transaction";
+      LOG(INFO) << "Cannot rollback a non-existing transaction";
       throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
     }
     else
@@ -281,19 +277,19 @@ namespace OrthancDatabases
   }
 
 
-  DatabaseManager::Transaction::Transaction(DatabaseManager& manager) :
-    lock_(manager.mutex_),
+  DatabaseManager::Transaction::Transaction(DatabaseManager& manager,
+                                            TransactionType type) :
     manager_(manager),
     database_(manager.GetDatabase()),
-    committed_(false)
+    active_(true)
   {
-    manager_.StartTransaction();
+    manager_.StartTransaction(type);
   }
 
 
   DatabaseManager::Transaction::~Transaction()
   {
-    if (!committed_)
+    if (active_)
     {
       try
       {
@@ -310,14 +306,28 @@ namespace OrthancDatabases
   
   void DatabaseManager::Transaction::Commit()
   {
-    if (committed_)
+    if (active_)
     {
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
+      manager_.CommitTransaction();
+      active_ = false;
     }
     else
     {
-      manager_.CommitTransaction();
-      committed_ = true;
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
+    }
+  }
+
+
+  void DatabaseManager::Transaction::Rollback()
+  {
+    if (active_)
+    {
+      manager_.RollbackTransaction();
+      active_ = false;
+    }
+    else
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
     }
   }
 
@@ -336,7 +346,7 @@ namespace OrthancDatabases
 
   void DatabaseManager::StatementBase::SetQuery(Query* query)
   {
-    std::auto_ptr<Query> protection(query);
+    std::unique_ptr<Query> protection(query);
     
     if (query_.get() != NULL)
     {
@@ -355,7 +365,7 @@ namespace OrthancDatabases
   
   void DatabaseManager::StatementBase::SetResult(IResult* result)
   {
-    std::auto_ptr<IResult> protection(result);
+    std::unique_ptr<IResult> protection(result);
     
     if (result_.get() != NULL)
     {
@@ -374,7 +384,6 @@ namespace OrthancDatabases
   
   DatabaseManager::StatementBase::StatementBase(DatabaseManager& manager) :
     manager_(manager),
-    lock_(manager_.mutex_),
     transaction_(manager_.GetTransaction())
   {
   }
@@ -475,7 +484,72 @@ namespace OrthancDatabases
       manager_.CloseIfUnavailable(e.GetErrorCode());
       throw;
     }
-  }  
+  }
+
+
+  int64_t DatabaseManager::StatementBase::ReadInteger64(size_t field) const
+  {
+    if (IsDone())
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_Database);
+    }
+    else
+    {
+      const IValue& value = GetResultField(field);
+      
+      switch (value.GetType())
+      {
+        case ValueType_Integer64:
+          return dynamic_cast<const Integer64Value&>(value).GetValue();
+
+        default:
+          //LOG(ERROR) << value.Format();
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      }
+    }
+  }
+
+
+  int32_t DatabaseManager::StatementBase::ReadInteger32(size_t field) const
+  {
+    if (IsDone())
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_Database);
+    }
+    else
+    {
+      int64_t value = ReadInteger64(field);
+
+      if (value != static_cast<int64_t>(static_cast<int32_t>(value)))
+      {
+        LOG(ERROR) << "Integer overflow";
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      }
+      else
+      {
+        return static_cast<int32_t>(value);
+      }
+    }
+  }
+
+    
+  std::string DatabaseManager::StatementBase::ReadString(size_t field) const
+  {
+    const IValue& value = GetResultField(field);
+
+    switch (value.GetType())
+    {
+      case ValueType_BinaryString:
+        return dynamic_cast<const BinaryStringValue&>(value).GetContent();
+
+      case ValueType_Utf8String:
+        return dynamic_cast<const Utf8StringValue&>(value).GetContent();
+
+      default:
+        //LOG(ERROR) << value.Format();
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+    }
+  }
   
   
   DatabaseManager::CachedStatement::CachedStatement(const StatementLocation& location,
@@ -502,7 +576,7 @@ namespace OrthancDatabases
   {
     try
     {
-      std::auto_ptr<Query> query(ReleaseQuery());
+      std::unique_ptr<Query> query(ReleaseQuery());
       
       if (query.get() != NULL)
       {
@@ -554,7 +628,7 @@ namespace OrthancDatabases
   {
     try
     {
-      std::auto_ptr<Query> query(ReleaseQuery());
+      std::unique_ptr<Query> query(ReleaseQuery());
       assert(query.get() != NULL);
 
       // The "statement_" object must be kept as long as the "IResult"
