@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -27,9 +27,10 @@ namespace OrthancDatabases
   enum ValueType
   {
     ValueType_BinaryString,
-    ValueType_File,
+    ValueType_InputFile,
     ValueType_Integer64,
     ValueType_Null,
+    ValueType_ResultFile,
     ValueType_Utf8String
   };
 
@@ -37,6 +38,14 @@ namespace OrthancDatabases
   {
     Dialect_MySQL,
     Dialect_PostgreSQL,
-    Dialect_SQLite
+    Dialect_SQLite,
+    Dialect_Unknown
+  };
+
+  enum TransactionType
+  {
+    TransactionType_ReadWrite,
+    TransactionType_ReadOnly,  // Should only arise with Orthanc SDK >= 1.9.2 in the index plugin
+    TransactionType_Implicit   // Should only arise with Orthanc SDK <= 1.9.1
   };
 }
