@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -21,8 +21,9 @@
 
 #include "ImplicitTransaction.h"
 
-#include <Core/Logging.h>
-#include <Core/OrthancException.h>
+#include <Compatibility.h>  // For std::unique_ptr<>
+#include <Logging.h>
+#include <OrthancException.h>
 
 #include <memory>
 
@@ -32,8 +33,7 @@ namespace OrthancDatabases
   
   
   ImplicitTransaction::ImplicitTransaction() :
-    state_(State_Ready),
-    readOnly_(true)
+    state_(State_Ready)
   {
   }
 
@@ -124,12 +124,7 @@ namespace OrthancDatabases
                                         const Dictionary& parameters)
   {
     CheckStateForExecution();    
-    std::auto_ptr<IResult> result(ExecuteInternal(statement, parameters));
-
-    if (!statement.IsReadOnly())
-    {
-      readOnly_ = false;
-    }
+    std::unique_ptr<IResult> result(ExecuteInternal(statement, parameters));
 
     state_ = State_Executed;
     return result.release();
@@ -142,11 +137,6 @@ namespace OrthancDatabases
     CheckStateForExecution();    
     ExecuteWithoutResultInternal(statement, parameters);
 
-    if (!statement.IsReadOnly())
-    {
-      readOnly_ = false;
-    }
-    
     state_ = State_Executed;
   }
 
