@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -23,21 +23,18 @@
 
 #include "IValue.h"
 
+#include <Compatibility.h>
+
 namespace OrthancDatabases
 {
   class NullValue : public IValue
   {
   public:
-    virtual ValueType GetType() const
+    virtual ValueType GetType() const ORTHANC_OVERRIDE
     {
       return ValueType_Null;
     }
 
-    virtual IValue* Convert(ValueType target) const;
-
-    virtual std::string Format() const
-    {
-      return "(null)";
-    }
+    virtual IValue* Convert(ValueType target) const ORTHANC_OVERRIDE;
   };
 }
