@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -26,8 +26,9 @@
 #include "../Common/NullValue.h"
 #include "../Common/Utf8StringValue.h"
 
-#include <Core/Logging.h>
-#include <Core/OrthancException.h>
+#include <Compatibility.h>  // For std::unique_ptr<>
+#include <Logging.h>
+#include <OrthancException.h>
 
 #include <cassert>
 #include <memory>
@@ -66,7 +67,7 @@ namespace OrthancDatabases
           sourceType != ValueType_Null &&
           sourceType != targetType)
       {
-        std::auto_ptr<IValue> converted(fields_[i]->Convert(targetType));
+        std::unique_ptr<IValue> converted(fields_[i]->Convert(targetType));
         
         if (converted.get() == NULL)
         {
