@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -23,7 +23,7 @@
 #include "PostgreSQLStatement.h"
 
 #include "../Common/BinaryStringValue.h"
-#include "../Common/FileValue.h"
+#include "../Common/InputFileValue.h"
 #include "../Common/Integer64Value.h"
 #include "../Common/NullValue.h"
 #include "../Common/ResultBase.h"
@@ -201,7 +201,7 @@ namespace OrthancDatabases
       // "Although there is no libpq function for deleting a
       // prepared statement, the SQL DEALLOCATE statement can be
       // used for that purpose."
-      database_.Execute("DEALLOCATE \"" + id_ + "\"");
+      database_.ExecuteMultiLines("DEALLOCATE \"" + id_ + "\"");
     }
 
     id_.clear();
@@ -278,7 +278,20 @@ namespace OrthancDatabases
                               1);
     }
 
-    if (result == NULL)
+    if (PQtransactionStatus(reinterpret_cast<PGconn*>(database_.pg_)) == PQTRANS_INERROR)
+    {
+      if (result != NULL)
+      {
+        PQclear(result);
+      }
+      
+#if ORTHANC_PLUGINS_VERSION_IS_ABOVE(1, 9, 2)
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_DatabaseCannotSerialize);
+#else
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_Database, "Collision between multiple writers");
+#endif
+    }
+    else if (result == NULL)
     {
       database_.ThrowException(true);
     }
@@ -288,10 +301,8 @@ namespace OrthancDatabases
 
 
   PostgreSQLStatement::PostgreSQLStatement(PostgreSQLDatabase& database,
-                                           const std::string& sql,
-                                           bool readOnly) :
+                                           const std::string& sql) :
     database_(database),
-    readOnly_(readOnly),
     sql_(sql),
     inputs_(new Inputs),
     formatter_(Dialect_PostgreSQL)
@@ -303,7 +314,6 @@ namespace OrthancDatabases
   PostgreSQLStatement::PostgreSQLStatement(PostgreSQLDatabase& database,
                                            const Query& query) :
     database_(database),
-    readOnly_(query.IsReadOnly()),
     inputs_(new Inputs),
     formatter_(Dialect_PostgreSQL)
   {
@@ -326,7 +336,7 @@ namespace OrthancDatabases
           DeclareInputBinary(i);
           break;
 
-        case ValueType_File:
+        case ValueType_InputFile:
           DeclareInputLargeObject(i);
           break;
 
@@ -476,7 +486,7 @@ namespace OrthancDatabases
     }
 
   public:
-    ResultWrapper(PostgreSQLStatement& statement) :
+    explicit ResultWrapper(PostgreSQLStatement& statement) :
       result_(new PostgreSQLResult(statement))
     {
       SetFieldsCount(result_->GetColumnsCount());
@@ -523,10 +533,10 @@ namespace OrthancDatabases
                      (parameters.GetValue(name)).GetContent());
           break;
 
-        case ValueType_File:
+        case ValueType_InputFile:
         {
-          const FileValue& blob =
-            dynamic_cast<const FileValue&>(parameters.GetValue(name));
+          const InputFileValue& blob =
+            dynamic_cast<const InputFileValue&>(parameters.GetValue(name));
 
           PostgreSQLLargeObject largeObject(database_, blob.GetContent());
           BindLargeObject(i, largeObject);
