@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -65,10 +65,19 @@ static PostgreSQLDatabase* CreateTestDatabase()
 
 static int64_t CountLargeObjects(PostgreSQLDatabase& db)
 {
-  // Count the number of large objects in the DB
-  PostgreSQLStatement s(db, "SELECT COUNT(*) FROM pg_catalog.pg_largeobject", true);
-  PostgreSQLResult r(s);
-  return r.GetInteger64(0);
+  PostgreSQLTransaction transaction(db, TransactionType_ReadOnly);
+
+  int64_t count;
+  
+  {
+    // Count the number of large objects in the DB
+    PostgreSQLStatement s(db, "SELECT COUNT(*) FROM pg_catalog.pg_largeobject");
+    PostgreSQLResult r(s);
+    count = r.GetInteger64(0);
+  }
+
+  transaction.Commit();
+  return count;
 }
 
 
@@ -77,10 +86,19 @@ TEST(PostgreSQL, Basic)
   std::unique_ptr<PostgreSQLDatabase> pg(CreateTestDatabase());
 
   ASSERT_FALSE(pg->DoesTableExist("Test"));
-  pg->Execute("CREATE TABLE Test(name INTEGER, value BIGINT)");
+  ASSERT_FALSE(pg->DoesColumnExist("Test", "value"));
+  ASSERT_FALSE(pg->DoesTableExist("TEST"));
+  ASSERT_FALSE(pg->DoesTableExist("test"));
+  pg->ExecuteMultiLines("CREATE TABLE Test(name INTEGER, value BIGINT)");
   ASSERT_TRUE(pg->DoesTableExist("Test"));
+  ASSERT_TRUE(pg->DoesTableExist("TEST"));
+  ASSERT_TRUE(pg->DoesTableExist("test"));
 
-  PostgreSQLStatement s(*pg, "INSERT INTO Test VALUES ($1,$2)", false);
+  ASSERT_TRUE(pg->DoesColumnExist("Test", "Value"));
+  ASSERT_TRUE(pg->DoesColumnExist("TEST", "VALUE"));
+  ASSERT_TRUE(pg->DoesColumnExist("test", "value"));
+  
+  PostgreSQLStatement s(*pg, "INSERT INTO Test VALUES ($1,$2)");
   s.DeclareInputInteger(0);
   s.DeclareInputInteger64(1);
 
@@ -99,7 +117,7 @@ TEST(PostgreSQL, Basic)
   s.Run();
 
   {
-    PostgreSQLStatement t(*pg, "SELECT name, value FROM Test ORDER BY name", true);
+    PostgreSQLStatement t(*pg, "SELECT name, value FROM Test ORDER BY name");
     PostgreSQLResult r(t);
 
     ASSERT_FALSE(r.IsDone());
@@ -121,7 +139,7 @@ TEST(PostgreSQL, Basic)
   }
 
   {
-    PostgreSQLStatement t(*pg, "SELECT name, value FROM Test WHERE name=$1", true);
+    PostgreSQLStatement t(*pg, "SELECT name, value FROM Test WHERE name=$1");
     t.DeclareInputInteger(0);
 
     {
@@ -149,9 +167,9 @@ TEST(PostgreSQL, String)
 {
   std::unique_ptr<PostgreSQLDatabase> pg(CreateTestDatabase());
 
-  pg->Execute("CREATE TABLE Test(name INTEGER, value VARCHAR(40))");
+  pg->ExecuteMultiLines("CREATE TABLE Test(name INTEGER, value VARCHAR(40))");
 
-  PostgreSQLStatement s(*pg, "INSERT INTO Test VALUES ($1,$2)", false);
+  PostgreSQLStatement s(*pg, "INSERT INTO Test VALUES ($1,$2)");
   s.DeclareInputInteger(0);
   s.DeclareInputString(1);
 
@@ -168,7 +186,7 @@ TEST(PostgreSQL, String)
   s.Run();
 
   {
-    PostgreSQLStatement t(*pg, "SELECT name, value FROM Test ORDER BY name", true);
+    PostgreSQLStatement t(*pg, "SELECT name, value FROM Test ORDER BY name");
     PostgreSQLResult r(t);
 
     ASSERT_FALSE(r.IsDone());
@@ -195,10 +213,10 @@ TEST(PostgreSQL, Transaction)
 {
   std::unique_ptr<PostgreSQLDatabase> pg(CreateTestDatabase());
 
-  pg->Execute("CREATE TABLE Test(name INTEGER, value INTEGER)");
+  pg->ExecuteMultiLines("CREATE TABLE Test(name INTEGER, value INTEGER)");
 
   {
-    PostgreSQLStatement s(*pg, "INSERT INTO Test VALUES ($1,$2)", false);
+    PostgreSQLStatement s(*pg, "INSERT INTO Test VALUES ($1,$2)");
     s.DeclareInputInteger(0);
     s.DeclareInputInteger(1);
     s.BindInteger(0, 42);
@@ -206,7 +224,15 @@ TEST(PostgreSQL, Transaction)
     s.Run();
 
     {
-      PostgreSQLTransaction t(*pg);
+      PostgreSQLTransaction t(*pg, TransactionType_ReadOnly);
+      s.BindInteger(0, 0);
+      s.BindInteger(1, 1);
+      // Failure, as INSERT in a read-only transaction
+      ASSERT_THROW(s.Run(), Orthanc::OrthancException);
+    }
+
+    {
+      PostgreSQLTransaction t(*pg, TransactionType_ReadWrite);
       s.BindInteger(0, 43);
       s.BindInteger(1, 4343);
       s.Run();
@@ -214,7 +240,7 @@ TEST(PostgreSQL, Transaction)
       s.BindInteger(1, 4444);
       s.Run();
 
-      PostgreSQLStatement u(*pg, "SELECT COUNT(*) FROM Test", true);
+      PostgreSQLStatement u(*pg, "SELECT COUNT(*) FROM Test");
       PostgreSQLResult r(u);
       ASSERT_EQ(3, r.GetInteger64(0));
 
@@ -222,13 +248,14 @@ TEST(PostgreSQL, Transaction)
     }
 
     {
-      PostgreSQLStatement u(*pg, "SELECT COUNT(*) FROM Test", true);
+      // Implicit transaction
+      PostgreSQLStatement u(*pg, "SELECT COUNT(*) FROM Test");
       PostgreSQLResult r(u);
       ASSERT_EQ(1, r.GetInteger64(0));  // Just "1" because of implicit rollback
     }
     
     {
-      PostgreSQLTransaction t(*pg);
+      PostgreSQLTransaction t(*pg, TransactionType_ReadWrite);
       s.BindInteger(0, 43);
       s.BindInteger(1, 4343);
       s.Run();
@@ -237,7 +264,7 @@ TEST(PostgreSQL, Transaction)
       s.Run();
 
       {
-        PostgreSQLStatement u(*pg, "SELECT COUNT(*) FROM Test", true);
+        PostgreSQLStatement u(*pg, "SELECT COUNT(*) FROM Test");
         PostgreSQLResult r(u);
         ASSERT_EQ(3, r.GetInteger64(0));
 
@@ -248,7 +275,8 @@ TEST(PostgreSQL, Transaction)
     }
 
     {
-      PostgreSQLStatement u(*pg, "SELECT COUNT(*) FROM Test", true);
+      PostgreSQLTransaction t(*pg, TransactionType_ReadOnly);
+      PostgreSQLStatement u(*pg, "SELECT COUNT(*) FROM Test");
       PostgreSQLResult r(u);
       ASSERT_EQ(3, r.GetInteger64(0));
     }
@@ -264,19 +292,19 @@ TEST(PostgreSQL, LargeObject)
   std::unique_ptr<PostgreSQLDatabase> pg(CreateTestDatabase());
   ASSERT_EQ(0, CountLargeObjects(*pg));
 
-  pg->Execute("CREATE TABLE Test(name VARCHAR, value OID)");
+  pg->ExecuteMultiLines("CREATE TABLE Test(name VARCHAR, value OID)");
 
   // Automatically remove the large objects associated with the table
-  pg->Execute("CREATE RULE TestDelete AS ON DELETE TO Test DO SELECT lo_unlink(old.value);");
+  pg->ExecuteMultiLines("CREATE RULE TestDelete AS ON DELETE TO Test DO SELECT lo_unlink(old.value);");
 
   {
-    PostgreSQLStatement s(*pg, "INSERT INTO Test VALUES ($1,$2)", false);
+    PostgreSQLStatement s(*pg, "INSERT INTO Test VALUES ($1,$2)");
     s.DeclareInputString(0);
     s.DeclareInputLargeObject(1);
     
     for (int i = 0; i < 10; i++)
     {
-      PostgreSQLTransaction t(*pg);
+      PostgreSQLTransaction t(*pg, TransactionType_ReadWrite);
 
       std::string value = "Value " + boost::lexical_cast<std::string>(i * 2);
       PostgreSQLLargeObject obj(*pg, value);
@@ -286,7 +314,7 @@ TEST(PostgreSQL, LargeObject)
       s.Run();
 
       std::string tmp;
-      PostgreSQLLargeObject::Read(tmp, *pg, obj.GetOid());
+      PostgreSQLLargeObject::ReadWhole(tmp, *pg, obj.GetOid());
       ASSERT_EQ(value, tmp);
 
       t.Commit();
@@ -297,8 +325,8 @@ TEST(PostgreSQL, LargeObject)
   ASSERT_EQ(10, CountLargeObjects(*pg));
 
   {
-    PostgreSQLTransaction t(*pg);
-    PostgreSQLStatement s(*pg, "SELECT * FROM Test ORDER BY name DESC", true);
+    PostgreSQLTransaction t(*pg, TransactionType_ReadOnly);
+    PostgreSQLStatement s(*pg, "SELECT * FROM Test ORDER BY name DESC");
     PostgreSQLResult r(s);
 
     ASSERT_FALSE(r.IsDone());
@@ -307,7 +335,7 @@ TEST(PostgreSQL, LargeObject)
     ASSERT_EQ("Index 9", r.GetString(0));
 
     std::string data;
-    r.GetLargeObject(data, 1);
+    r.GetLargeObjectContent(data, 1);
     ASSERT_EQ("Value 18", data);    
 
     r.Next();
@@ -318,8 +346,8 @@ TEST(PostgreSQL, LargeObject)
 
 
   {
-    PostgreSQLTransaction t(*pg);
-    PostgreSQLStatement s(*pg, "DELETE FROM Test WHERE name='Index 9'", false);
+    PostgreSQLTransaction t(*pg, TransactionType_ReadWrite);
+    PostgreSQLStatement s(*pg, "DELETE FROM Test WHERE name='Index 9'");
     s.Run();
     t.Commit();
   }
@@ -327,8 +355,8 @@ TEST(PostgreSQL, LargeObject)
 
   {
     // Count the number of items in the DB
-    PostgreSQLTransaction t(*pg);
-    PostgreSQLStatement s(*pg, "SELECT COUNT(*) FROM Test", true);
+    PostgreSQLTransaction t(*pg, TransactionType_ReadOnly);
+    PostgreSQLStatement s(*pg, "SELECT COUNT(*) FROM Test");
     PostgreSQLResult r(s);
     ASSERT_EQ(9, r.GetInteger64(0));
   }
@@ -339,59 +367,125 @@ TEST(PostgreSQL, LargeObject)
 
 TEST(PostgreSQL, StorageArea)
 {
-  PostgreSQLStorageArea storageArea(globalParameters_);
-  storageArea.SetClearAll(true);
+  std::unique_ptr<PostgreSQLDatabase> database(PostgreSQLDatabase::CreateDatabaseConnection(globalParameters_));
+  
+  PostgreSQLStorageArea storageArea(globalParameters_, true /* clear database */);
 
   {
-    DatabaseManager::Transaction transaction(storageArea.GetManager());
-    PostgreSQLDatabase& db = 
-      dynamic_cast<PostgreSQLDatabase&>(transaction.GetDatabase());
-
-    ASSERT_EQ(0, CountLargeObjects(db));
+    std::unique_ptr<OrthancDatabases::StorageBackend::IAccessor> accessor(storageArea.CreateAccessor());
+    
+    ASSERT_EQ(0, CountLargeObjects(*database));
   
     for (int i = 0; i < 10; i++)
     {
       std::string uuid = boost::lexical_cast<std::string>(i);
       std::string value = "Value " + boost::lexical_cast<std::string>(i * 2);
-      storageArea.Create(transaction, uuid, value.c_str(), value.size(), OrthancPluginContentType_Unknown);
+      accessor->Create(uuid, value.c_str(), value.size(), OrthancPluginContentType_Unknown);
     }
 
-    std::string tmp;
-    ASSERT_THROW(storageArea.ReadToString(tmp, transaction, "nope", OrthancPluginContentType_Unknown), 
+    std::string buffer;
+    ASSERT_THROW(OrthancDatabases::StorageBackend::ReadWholeToString(buffer, *accessor, "nope", OrthancPluginContentType_Unknown), 
                  Orthanc::OrthancException);
   
-    ASSERT_EQ(10, CountLargeObjects(db));
-    storageArea.Remove(transaction, "5", OrthancPluginContentType_Unknown);
+    ASSERT_EQ(10, CountLargeObjects(*database));
+    accessor->Remove("5", OrthancPluginContentType_Unknown);
 
-    ASSERT_EQ(9, CountLargeObjects(db));
+    ASSERT_EQ(9, CountLargeObjects(*database));
 
     for (int i = 0; i < 10; i++)
     {
       std::string uuid = boost::lexical_cast<std::string>(i);
       std::string expected = "Value " + boost::lexical_cast<std::string>(i * 2);
-      std::string content;
 
       if (i == 5)
       {
-        ASSERT_THROW(storageArea.ReadToString(content, transaction, uuid, OrthancPluginContentType_Unknown), 
+        ASSERT_THROW(OrthancDatabases::StorageBackend::ReadWholeToString(buffer, *accessor, uuid, OrthancPluginContentType_Unknown), 
                      Orthanc::OrthancException);
       }
       else
       {
-        storageArea.ReadToString(content, transaction, uuid, OrthancPluginContentType_Unknown);
-        ASSERT_EQ(expected, content);
+        OrthancDatabases::StorageBackend::ReadWholeToString(buffer, *accessor, uuid, OrthancPluginContentType_Unknown);
+        ASSERT_EQ(expected, buffer);
       }
     }
 
     for (int i = 0; i < 10; i++)
     {
-      storageArea.Remove(transaction, boost::lexical_cast<std::string>(i),
-                         OrthancPluginContentType_Unknown);
+      accessor->Remove(boost::lexical_cast<std::string>(i), OrthancPluginContentType_Unknown);
     }
 
-    ASSERT_EQ(0, CountLargeObjects(db));
+    ASSERT_EQ(0, CountLargeObjects(*database));
+  }
+}
 
-    transaction.Commit();
+
+TEST(PostgreSQL, StorageReadRange)
+{
+  std::unique_ptr<OrthancDatabases::PostgreSQLDatabase> database(
+    OrthancDatabases::PostgreSQLDatabase::CreateDatabaseConnection(globalParameters_));
+  
+  OrthancDatabases::PostgreSQLStorageArea storageArea(globalParameters_, true /* clear database */);
+
+  {
+    std::unique_ptr<OrthancDatabases::StorageBackend::IAccessor> accessor(storageArea.CreateAccessor());
+    ASSERT_EQ(0, CountLargeObjects(*database));
+    accessor->Create("uuid", "abcd\0\1\2\3\4\5", 10, OrthancPluginContentType_Unknown);
+    ASSERT_EQ(1u, CountLargeObjects(*database));
+  }
+
+  {
+    std::unique_ptr<OrthancDatabases::StorageBackend::IAccessor> accessor(storageArea.CreateAccessor());
+    ASSERT_EQ(1u, CountLargeObjects(*database));
+
+    std::string s;
+    OrthancDatabases::StorageBackend::ReadWholeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown);
+    ASSERT_EQ(10u, s.size());
+    ASSERT_EQ('a', s[0]);
+    ASSERT_EQ('d', s[3]);
+    ASSERT_EQ('\0', s[4]);
+    ASSERT_EQ('\5', s[9]);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 0, 0);
+    ASSERT_TRUE(s.empty());
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 0, 1);
+    ASSERT_EQ(1u, s.size());
+    ASSERT_EQ('a', s[0]);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 4, 1);
+    ASSERT_EQ(1u, s.size());
+    ASSERT_EQ('\0', s[0]);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 9, 1);
+    ASSERT_EQ(1u, s.size());
+    ASSERT_EQ('\5', s[0]);
+
+    // Cannot read non-empty range after the end of the string. NB:
+    // The behavior on range (10, 0) is different than in MySQL!
+    ASSERT_THROW(OrthancDatabases::StorageBackend::ReadRangeToString(
+                   s, *accessor, "uuid", OrthancPluginContentType_Unknown, 10, 0), Orthanc::OrthancException);
+
+    ASSERT_THROW(OrthancDatabases::StorageBackend::ReadRangeToString(
+                   s, *accessor, "uuid", OrthancPluginContentType_Unknown, 10, 1), Orthanc::OrthancException);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 0, 4);
+    ASSERT_EQ(4u, s.size());
+    ASSERT_EQ('a', s[0]);
+    ASSERT_EQ('b', s[1]);
+    ASSERT_EQ('c', s[2]);
+    ASSERT_EQ('d', s[3]);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 4, 6);
+    ASSERT_EQ(6u, s.size());
+    ASSERT_EQ('\0', s[0]);
+    ASSERT_EQ('\1', s[1]);
+    ASSERT_EQ('\2', s[2]);
+    ASSERT_EQ('\3', s[3]);
+    ASSERT_EQ('\4', s[4]);
+    ASSERT_EQ('\5', s[5]);
+
+    ASSERT_THROW(OrthancDatabases::StorageBackend::ReadRangeToString(
+                   s, *accessor, "uuid", OrthancPluginContentType_Unknown, 4, 7), Orthanc::OrthancException);
   }
 }
 
@@ -404,7 +498,7 @@ TEST(PostgreSQL, ImplicitTransaction)
   ASSERT_FALSE(db->DoesTableExist("test2"));
 
   {
-    std::unique_ptr<OrthancDatabases::ITransaction> t(db->CreateTransaction(false));
+    std::unique_ptr<OrthancDatabases::ITransaction> t(db->CreateTransaction(TransactionType_ReadWrite));
     ASSERT_FALSE(t->IsImplicit());
   }
 
@@ -412,7 +506,7 @@ TEST(PostgreSQL, ImplicitTransaction)
     Query query("CREATE TABLE test(id INT)", false);
     std::unique_ptr<IPrecompiledStatement> s(db->Compile(query));
     
-    std::unique_ptr<ITransaction> t(db->CreateTransaction(true));
+    std::unique_ptr<ITransaction> t(db->CreateTransaction(TransactionType_Implicit));
     ASSERT_TRUE(t->IsImplicit());
     ASSERT_THROW(t->Commit(), Orthanc::OrthancException);
     ASSERT_THROW(t->Rollback(), Orthanc::OrthancException);
@@ -430,7 +524,7 @@ TEST(PostgreSQL, ImplicitTransaction)
     Query query("CREATE TABLE test2(id INT)", false);
     std::unique_ptr<IPrecompiledStatement> s(db->Compile(query));
     
-    std::unique_ptr<ITransaction> t(db->CreateTransaction(true));
+    std::unique_ptr<ITransaction> t(db->CreateTransaction(TransactionType_Implicit));
 
     Dictionary args;
     t->ExecuteWithoutResult(*s, args);
@@ -444,34 +538,35 @@ TEST(PostgreSQL, ImplicitTransaction)
 #if ORTHANC_PLUGINS_HAS_DATABASE_CONSTRAINT == 1
 TEST(PostgreSQLIndex, CreateInstance)
 {
-  OrthancDatabases::PostgreSQLIndex db(globalParameters_);
+  OrthancDatabases::PostgreSQLIndex db(NULL, globalParameters_);
   db.SetClearAll(true);
-  db.Open();
+
+  std::unique_ptr<OrthancDatabases::DatabaseManager> manager(OrthancDatabases::IndexBackend::CreateSingleDatabaseManager(db));
 
   std::string s;
-  ASSERT_TRUE(db.LookupGlobalProperty(s, Orthanc::GlobalProperty_DatabaseInternal1));
+  ASSERT_TRUE(db.LookupGlobalProperty(s, *manager, MISSING_SERVER_IDENTIFIER, Orthanc::GlobalProperty_DatabaseInternal1));
   ASSERT_EQ("2", s);
 
   OrthancPluginCreateInstanceResult r1, r2;
   
   memset(&r1, 0, sizeof(r1));
-  db.CreateInstance(r1, "a", "b", "c", "d");
+  db.CreateInstance(r1, *manager, "a", "b", "c", "d");
   ASSERT_TRUE(r1.isNewInstance);
   ASSERT_TRUE(r1.isNewSeries);
   ASSERT_TRUE(r1.isNewStudy);
   ASSERT_TRUE(r1.isNewPatient);
 
   memset(&r2, 0, sizeof(r2));
-  db.CreateInstance(r2, "a", "b", "c", "d");
+  db.CreateInstance(r2, *manager, "a", "b", "c", "d");
   ASSERT_FALSE(r2.isNewInstance);
   ASSERT_EQ(r1.instanceId, r2.instanceId);
 
   // Breaking the hierarchy
   memset(&r2, 0, sizeof(r2));
-  ASSERT_THROW(db.CreateInstance(r2, "a", "e", "c", "f"), Orthanc::OrthancException);
+  ASSERT_THROW(db.CreateInstance(r2, *manager, "a", "e", "c", "f"), Orthanc::OrthancException);
 
   memset(&r2, 0, sizeof(r2));
-  db.CreateInstance(r2, "a", "b", "c", "e");
+  db.CreateInstance(r2, *manager, "a", "b", "c", "e");
   ASSERT_TRUE(r2.isNewInstance);
   ASSERT_FALSE(r2.isNewSeries);
   ASSERT_FALSE(r2.isNewStudy);
@@ -482,7 +577,7 @@ TEST(PostgreSQLIndex, CreateInstance)
   ASSERT_NE(r1.instanceId, r2.instanceId);
 
   memset(&r2, 0, sizeof(r2));
-  db.CreateInstance(r2, "a", "b", "f", "g");
+  db.CreateInstance(r2, *manager, "a", "b", "f", "g");
   ASSERT_TRUE(r2.isNewInstance);
   ASSERT_TRUE(r2.isNewSeries);
   ASSERT_FALSE(r2.isNewStudy);
@@ -493,7 +588,7 @@ TEST(PostgreSQLIndex, CreateInstance)
   ASSERT_NE(r1.instanceId, r2.instanceId);
 
   memset(&r2, 0, sizeof(r2));
-  db.CreateInstance(r2, "a", "h", "i", "j");
+  db.CreateInstance(r2, *manager, "a", "h", "i", "j");
   ASSERT_TRUE(r2.isNewInstance);
   ASSERT_TRUE(r2.isNewSeries);
   ASSERT_TRUE(r2.isNewStudy);
@@ -504,7 +599,7 @@ TEST(PostgreSQLIndex, CreateInstance)
   ASSERT_NE(r1.instanceId, r2.instanceId);
 
   memset(&r2, 0, sizeof(r2));
-  db.CreateInstance(r2, "k", "l", "m", "n");
+  db.CreateInstance(r2, *manager, "k", "l", "m", "n");
   ASSERT_TRUE(r2.isNewInstance);
   ASSERT_TRUE(r2.isNewSeries);
   ASSERT_TRUE(r2.isNewStudy);
@@ -520,7 +615,6 @@ TEST(PostgreSQLIndex, CreateInstance)
 TEST(PostgreSQL, Lock2)
 {
   std::unique_ptr<PostgreSQLDatabase> db1(CreateTestDatabase());
-  db1->Open();
 
   ASSERT_FALSE(db1->ReleaseAdvisoryLock(43)); // lock counter = 0
   ASSERT_TRUE(db1->AcquireAdvisoryLock(43));  // lock counter = 1
@@ -536,7 +630,6 @@ TEST(PostgreSQL, Lock2)
 
   {
     std::unique_ptr<PostgreSQLDatabase> db2(CreateTestDatabase());
-    db2->Open();
 
     // The "db1" is still actively locking
     ASSERT_FALSE(db2->AcquireAdvisoryLock(43));
