@@ -4,9 +4,8 @@
 
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -75,7 +74,6 @@ extern "C"
     _OrthancPluginDatabaseAnswerType_Int64 = 15,
     _OrthancPluginDatabaseAnswerType_Resource = 16,
     _OrthancPluginDatabaseAnswerType_String = 17,
-    _OrthancPluginDatabaseAnswerType_MatchingResource = 18,  /* New in Orthanc 1.5.2 */
 
     _OrthancPluginDatabaseAnswerType_INTERNAL = 0x7fffffff
   } _OrthancPluginDatabaseAnswerType;
@@ -121,55 +119,6 @@ extern "C"
     const char*                sopInstanceUid;
   } OrthancPluginExportedResource;
 
-  typedef struct   /* New in Orthanc 1.5.2 */
-  {
-    OrthancPluginResourceType    level;
-    uint16_t                     tagGroup;
-    uint16_t                     tagElement;
-    uint8_t                      isIdentifierTag;
-    uint8_t                      isCaseSensitive;
-    uint8_t                      isMandatory;
-    OrthancPluginConstraintType  type;
-    uint32_t                     valuesCount;
-    const char* const*           values;
-  } OrthancPluginDatabaseConstraint;
-
-  typedef struct   /* New in Orthanc 1.5.2 */
-  {
-    const char*  resourceId;
-    const char*  someInstanceId;  /* Can be NULL if not requested */
-  } OrthancPluginMatchingResource;
-
-  typedef struct   /* New in Orthanc 1.5.2 */
-  {
-    /* Mandatory field */
-    uint8_t  isNewInstance;
-    int64_t  instanceId;
-
-    /* The following fields must only be set if "isNewInstance" is "true" */
-    uint8_t  isNewPatient;
-    uint8_t  isNewStudy;
-    uint8_t  isNewSeries;
-    int64_t  patientId;
-    int64_t  studyId;
-    int64_t  seriesId;
-  } OrthancPluginCreateInstanceResult;
-
-  typedef struct  /* New in Orthanc 1.5.2 */
-  {
-    int64_t      resource;
-    uint16_t     group;
-    uint16_t     element;
-    const char*  value;
-  } OrthancPluginResourcesContentTags;
-    
-  typedef struct  /* New in Orthanc 1.5.2 */
-  {
-    int64_t      resource;
-    int32_t      metadata;
-    const char*  value;
-  } OrthancPluginResourcesContentMetadata;
-
 
   typedef struct
   {
@@ -322,19 +271,6 @@ extern "C"
     context->InvokeService(context, _OrthancPluginService_DatabaseAnswer, &params);
   }
 
-  ORTHANC_PLUGIN_INLINE void OrthancPluginDatabaseAnswerMatchingResource(
-    OrthancPluginContext*                 context,
-    OrthancPluginDatabaseContext*         database,
-    const OrthancPluginMatchingResource*  match)
-  {
-    _OrthancPluginDatabaseAnswer params;
-    memset(&params, 0, sizeof(params));
-    params.database = database;
-    params.type = _OrthancPluginDatabaseAnswerType_MatchingResource;
-    params.valueGeneric = match;
-    context->InvokeService(context, _OrthancPluginService_DatabaseAnswer, &params);
-  }
-
   ORTHANC_PLUGIN_INLINE void OrthancPluginDatabaseSignalDeletedAttachment(
     OrthancPluginContext*          context,
     OrthancPluginDatabaseContext*  database,
@@ -701,10 +637,6 @@ extern "C"
 
   typedef struct
   {
-    /**
-     * Base extensions since Orthanc 1.0.0
-     **/
-    
     /* Output: Use OrthancPluginDatabaseAnswerString() */
     OrthancPluginErrorCode  (*getAllPublicIdsWithLimit) (
       /* outputs */
@@ -749,83 +681,7 @@ extern "C"
       OrthancPluginResourceType resourceType,
       const OrthancPluginDicomTag* tag,
       OrthancPluginIdentifierConstraint constraint);
-
-
-    /**
-     * Extensions since Orthanc 1.4.0
-     **/
-    
-    /* Output: Use OrthancPluginDatabaseAnswerInt64() */
-    OrthancPluginErrorCode  (*lookupIdentifierRange) (
-      /* outputs */
-      OrthancPluginDatabaseContext* context,
-      /* inputs */
-      void* payload,
-      OrthancPluginResourceType resourceType,
-      uint16_t group,
-      uint16_t element,
-      const char* start,
-      const char* end);
-
-    
-    /**
-     * Extensions since Orthanc 1.5.2
-     **/
-    
-    /* Ouput: Use OrthancPluginDatabaseAnswerMatchingResource */
-    OrthancPluginErrorCode  (*lookupResources) (
-      /* outputs */
-      OrthancPluginDatabaseContext* context,
-      /* inputs */
-      void* payload,
-      uint32_t constraintsCount,
-      const OrthancPluginDatabaseConstraint* constraints,
-      OrthancPluginResourceType queryLevel,
-      uint32_t limit,
-      uint8_t requestSomeInstance);
-
-    
-    OrthancPluginErrorCode  (*createInstance) (
-      /* output */
-      OrthancPluginCreateInstanceResult* output,
-      /* inputs */
-      void* payload,
-      const char* hashPatient,
-      const char* hashStudy,
-      const char* hashSeries,
-      const char* hashInstance);
-
-    OrthancPluginErrorCode  (*setResourcesContent) (
-      /* inputs */
-      void* payload,
-      uint32_t countIdentifierTags,
-      const OrthancPluginResourcesContentTags* identifierTags,
-      uint32_t countMainDicomTags,
-      const OrthancPluginResourcesContentTags* mainDicomTags,
-      uint32_t countMetadata,
-      const OrthancPluginResourcesContentMetadata* metadata);
-
-    /* Ouput: Use OrthancPluginDatabaseAnswerString */
-    OrthancPluginErrorCode  (*getChildrenMetadata) (
-      /* outputs */
-      OrthancPluginDatabaseContext* context,
-      /* inputs */
-      void* payload,
-      int64_t resourceId,
-      int32_t metadata);
-
-    OrthancPluginErrorCode  (*getLastChangeIndex) (
-      /* outputs */
-      int64_t* target,
-      /* inputs */
-      void* payload);
-                   
-    OrthancPluginErrorCode  (*tagMostRecentPatient) (
-      /* inputs */
-      void* payload,
-      int64_t patientId);
-                   
-  } OrthancPluginDatabaseExtensions;
+   } OrthancPluginDatabaseExtensions;
 
 /*<! @endcond */
 
@@ -838,8 +694,13 @@ extern "C"
   } _OrthancPluginRegisterDatabaseBackend;
 
   /**
-   * Register a custom database back-end (for legacy plugins).
+   * Register a custom database back-end.
    *
+   * Instead of manually filling the OrthancPluginDatabaseBackend
+   * structure, you should instead implement a concrete C++ class
+   * deriving from ::OrthancPlugins::IDatabaseBackend, and register it
+   * using ::OrthancPlugins::DatabaseBackendAdapter::Register().
+   * 
    * @param context The Orthanc plugin context, as received by OrthancPluginInitialize().
    * @param backend The callbacks of the custom database engine.
    * @param payload Pointer containing private information for the database engine.
@@ -892,6 +753,11 @@ extern "C"
   /**
    * Register a custom database back-end.
    *
+   * Instead of manually filling the OrthancPluginDatabaseBackendV2
+   * structure, you should instead implement a concrete C++ class
+   * deriving from ::OrthancPlugins::IDatabaseBackend, and register it
+   * using ::OrthancPlugins::DatabaseBackendAdapter::Register().
+   * 
    * @param context The Orthanc plugin context, as received by OrthancPluginInitialize().
    * @param backend The callbacks of the custom database engine.
    * @param payload Pointer containing private information for the database engine.
